#include "icmp.h" 
#include <netdb.h> 


int sock;          

/* Calcula checksum */
u_short in_chksum(u_short *ptr, int nbytes)
{
  register long           sum;            /* assumes long == 32 bits */
  u_short                 oddbyte;
  register u_short        answer;         /* assumes u_short == 16 bits */

  /*
   * Our algorithm is simple, using a 32-bit accumulator (sum),
   * we add sequential 16-bit words to it, and at the end, fold back
   * all the carry bits from the top 16 bits into the lower 16 bits.
   */

  sum = 0;
  while (nbytes > 1)
  {
    sum += *ptr++;
    nbytes -= 2;
  }

  /* mop up an odd byte, if necessary */
  if (nbytes == 1)
  {
    oddbyte = 0;            /* make sure top half is zero */
    *((u_char *) &oddbyte) = *(u_char *)ptr;   /* one byte only */
    sum += oddbyte;
  }

  /*
   * Add back carry outs from top 16 bits to low 16 bits.
   */

  sum  = (sum >> 16) + (sum & 0xffff);    /* add high-16 to low-16 */
  sum += (sum >> 16);                     /* add carry */
  answer = ~sum;          /* ones-complement, then truncate to 16 bits */

  return((u_short) answer);
}
/* abre SOCK_RAW */

int opensock(void)
{
if((sock=socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) < 0 )
{	
	fprintf(stderr, "cant create SOCK_RAW socket\n");
	return -1;
}
}


int icmp_snd(char *dat, size_t len, struct sockaddr_in to)
{
unsigned char                packet[MAXPACKET];  
register struct icmp  *icmphdr;
int sent;      

bzero(packet,sizeof(packet));
 

  
icmphdr = (struct icmp *) packet;    
icmphdr->icmp_type = ICMP_ECHOREPLY;
bcopy(dat, icmphdr->icmp_data,len);
icmphdr->icmp_cksum = in_chksum((u_short *) icmphdr, len + ICMPHDRSIZE);
  
if((sent = sendto(sock, packet, len + ICMPHDRSIZE, 0, (struct sockaddr *) &to, sizeof(to))) < 0 )
{
	fprintf(stderr,"Error on sendto()\n");
	return -1;
}

}

/* recebe pacote icmp, retorna tamanho do icmp data recebido, e copia data para * data */
/* em erro retorna -1 */

int icmp_rec(char *data, size_t len, struct sockaddr_in from)
{
struct ip *iphdr;
struct icmp *icmphdr;
unsigned char rpacket[MAXPACKET];
int tam,iphl,clilen=sizeof(from);
  
  
while(1)
{
	bzero(rpacket, sizeof(rpacket));
	if((tam = recvfrom(sock, rpacket, MAXPACKET, 0, (struct sockaddr *) &from, &clilen)) < 0 )
		{
		fprintf(stderr,"Error on recvfrom()");
		return -1;
		}
	iphdr = (struct ip *) rpacket;
	iphl = iphdr->ip_hl << 2;    
	tam -= iphl;
	icmphdr = (struct icmp *) (rpacket + iphl);
	if(!icmphdr->icmp_id && !icmphdr->icmp_code && icmphdr->icmp_type == ICMP_ECHOREPLY)
		break;
}

tam -= ICMPHDRSIZE;
bcopy(icmphdr->icmp_data, data, tam);

return(tam);
}

u_long resolve(char *hostname)
{
struct in_addr addr;
struct hostent *ent;
  
if((addr.s_addr=inet_addr(hostname)) == -1) 
{
	if(!(ent=gethostbyname(hostname))) 
	{
		fprintf(stderr,"no such host %s\n",hostname);
		exit(0);
	}
bcopy(ent->h_addr,(char *)&addr.s_addr,ent->h_length);
}

  return addr.s_addr;
}

